use std::collections::HashMap;

use regex::Regex;

use crate::common::load_input;

struct Game {
    id: i32,
    played: Vec<i32>,
    winning: Vec<i32>,
}

fn nums_in_string(s: &str) -> Vec<i32>{
    let rx = Regex::new(r"(\d+)").unwrap();
    let result = rx.captures_iter(s);

    result.map(|e| e.get(0).unwrap().as_str().parse().unwrap()).collect()
}

fn parse_input(lines: &Vec<String>) -> Vec<Game> {
    
    return lines.iter().map(|line| {
        let (game_part, num_part) = line.split_once(":").unwrap();
        let game_id:i32 = game_part.split_at(5).1.trim().parse().unwrap();
        let (played_str, winning_str) = num_part.split_once("|").unwrap();

        return Game{
            id: game_id,
            played: nums_in_string(played_str),
            winning: nums_in_string(winning_str)
        };
    }).collect();
}

fn games_won(game: &Game)-> i32 {
    return game.played.iter().filter(|e| game.winning.contains(e)).count().try_into().unwrap();
}

fn p2_cards(game: &Game) -> Vec<usize> {
    let mut rtn: Vec<usize> = Vec::new();

    for i in 0..games_won(game){
        rtn.push((game.id + i+1) as usize);
    }

    return rtn;
}

fn p2_num_cards(games: &Vec<Game>, game: &Game, cache: &mut HashMap<usize,i32>) -> i32 {
    return match cache.get(&(game.id as usize)) {
        Some(value) => {*value} // Cache hit
        None =>{
            let cards = p2_cards(game);
            let mut sum = 1i32;
        
            for card in cards.iter() {
                match games.get(*card){
                    Some(value) => {
                        sum += p2_num_cards(games, value,cache);
                    },
                    None => {}
                }
            }
        
            cache.insert(game.id as usize, sum);

            sum
        }
    }
}

pub fn run_part2() -> Result<i32, Box<dyn std::error::Error>> {
    let lines = load_input("./input/4.txt")?;
    let games = parse_input(&lines);

    let mut lookup_table: HashMap<usize,i32> = HashMap::new();
    let mut sum = 0i32;
    
    for game in games.iter() {
        sum += p2_num_cards(&games, game, &mut lookup_table)
    }


    Ok(sum)
}

pub fn run() -> Result<i32, Box<dyn std::error::Error>> {
    let lines = load_input("./input/4.txt")?;
    let games = parse_input(&lines);
    let points = games.iter()
        .map(games_won)
        .filter(|e| *e > 0i32)
        .map(|e| 2i32.pow(e as u32 -1 ))
        .sum();
    Ok(points)
}