use std::str::FromStr;

use regex::Regex;

use crate::common::load_input;

extern crate regex;

fn num(line: String) -> i32 {
    let mut digits: String = String::from_str("").unwrap();

    for (_, c) in line.chars().enumerate() {
        if c >= '0' && c <= '9' {
            digits.push(c);
        }
    }

    return get_num_from_digits(digits);
}

fn num_part2(line: String) -> i32 {
    let mut last_digit: &str = "";
    let mut last_digit_index: usize = 0;

    let mut first_digit: &str = "";
    let mut first_digit_index: usize = usize::MAX;

    for pattern in [r"(one)",r"(two)",r"(three)",r"(four)",r"(five)",r"(six)",r"(seven)",r"(eight)",r"(nine)",r"(\d)"]{
        let regex = Regex::new(pattern).unwrap();
        let result = regex.captures_iter(&line);

        for mat in result {
            let match_str = mat.get(0).unwrap().as_str();
            let match_index = mat.get(0).unwrap().start();

            // Check first digit
            if match_index <= first_digit_index {
                first_digit_index = match_index;
                first_digit = match_str;
            }

            if match_index >= last_digit_index {
                last_digit_index = match_index;
                last_digit = match_str;
            }
        }
    }

    let mut picked_digits: String = String::from_str("").unwrap();

    if first_digit.len() > 1 {
        picked_digits.push_str(num_lookup(first_digit));
    }else{
        picked_digits.push_str(first_digit);
    }

    if last_digit.len() > 1 {
        picked_digits.push_str(num_lookup(last_digit));
    }else{
        picked_digits.push_str(last_digit);
    }

    return picked_digits.parse().unwrap();
}

/// Converts a written number into its digit
/// Panics if failed
fn num_lookup(str: &str) -> &str{
    match str {
        "one" => "1",
        "two" => "2",
        "three" => "3",
        "four" => "4",
        "five" => "5",
        "six" => "6",
        "seven" => "7",
        "eight" => "8",
        "nine" => "9",
        _ => panic!("Nan")
    }
}

fn get_num_from_digits(digits: String)-> i32{
    let mut picked_digits: String = String::from_str("").unwrap();

    picked_digits.push(digits.chars().nth(0).unwrap());
    picked_digits.push(digits.chars().last().unwrap());

    return picked_digits.parse().unwrap();
}

pub fn run_part2() -> Result<i32, Box<dyn std::error::Error>> {
    let lines = load_input("./input/1.txt")?;

    let mut sum = 0i32;

    for line in lines {
        sum += num_part2(line);
    }

    Ok(sum)
}

pub fn run() -> Result<i32, Box<dyn std::error::Error>> {
    let lines = load_input("./input/1.txt")?;

    let mut sum = 0i32;

    for line in lines {
        sum += num(line);
    }

    Ok(sum)
}
