use crate::common::load_input;


struct Game {
    id: i32,
    sets: Vec<Set>,
}

/// Red, Green, Blue
struct Set {
    red: i32,
    green: i32,
    blue: i32,
}

fn parse_line(line: &str) -> Game {
    let (game_part, sets_part) = line.split_once(":").unwrap();
    let sets = sets_part.split(";");

    let game = Game {
        id: game_part.split_at(5).1.parse().unwrap(),
        sets: sets
            .map(|set| {
                let mut rtn = Set {
                    red: 0,
                    green: 0,
                    blue: 0,
                };
                set.split(",").for_each(|color| {
                    if color.contains("red") {
                        rtn.red = color.split_at(color.len() - 4).0.trim().parse().unwrap();
                        return;
                    }
                    if color.contains("green") {
                        rtn.green = color.split_at(color.len() - 6).0.trim().parse().unwrap();
                        return;
                    }
                    if color.contains("blue") {
                        rtn.blue = color.split_at(color.len() - 5).0.trim().parse().unwrap();
                        return;
                    }
                });
                return rtn;
            })
            .collect(),
    };

    return game;
}

/// Returns a set with the maximum value for each color
fn max_set(sets: &Vec<Set>) -> Set {
    return Set {
        red: sets
            .iter()
            .max_by_key(|e| e.red)
            .unwrap_or(&Set {
                red: 0,
                green: 0,
                blue: 0,
            })
            .red,
        green: sets
            .iter()
            .max_by_key(|e| e.green)
            .unwrap_or(&Set {
                red: 0,
                green: 0,
                blue: 0,
            })
            .green,
        blue: sets
            .iter()
            .max_by_key(|e| e.blue)
            .unwrap_or(&Set {
                red: 0,
                green: 0,
                blue: 0,
            })
            .blue,
    };
}

pub fn run_part2() -> Result<i32, Box<dyn std::error::Error>> {
    let lines = load_input("./input/2.txt")?;

    let max = lines
        .iter()
        .map(|e| parse_line(e))
        .map(|game|{
            let max = max_set(&game.sets);
            return max.blue * max.green * max.red;
        }).sum();

    Ok(max)
}

pub fn run() -> Result<i32, Box<dyn std::error::Error>> {
    let lines = load_input("./input/2.txt")?;

    let sum_ids = lines
        .iter()
        .map(|e| parse_line(e))
        .filter(|game| {
            let max = max_set(&game.sets);
            return max.red <= 12 && max.green <= 13 && max.blue <= 14;
        })
        .map(|game| game.id)
        .sum();

    Ok(sum_ids)
}
